function [out]=spal(y,X,L,varargin)

%   SPAL is the main function of the Sparse Projection-based Adaptive
%   Learning toolbox.
%   This function, at this stage, is not optimized with respect to speed or
%   with respect to memory consumption. 
%
%   [out]= spal(y,X,L,@METHOD,options,'Option_name1',Option_value1,'Option_name2',Option_value2)
%
%   INPUT (mandatory):
%         y:  Observation vector (with N observations)
%
%         X  Either:
%             1) An NxL matrix in case that y=Ah.
%             2) A vector with N+L-1 components, in system identification
%             case.
%             3) ... other options will be added in forthcoming versions
%
%         L:  length of unknown system/signal h       
%
%   INPUT (Not all of them are needed)
%         @METHOD Either:
%             1) @APWL1 (For projections onto the weighted L1 ball)
%             2) @APL1  (For projections onto the L1 ball)
%         
%         options: A structure array with the options produced by spaloptions.m 
%        
%         'Option_name'- Option_value pairs as they are descibed in spaloptions.m (see spaloptions help)
%     -------------------------------------------------------------------------------
%    
%   OUTPUT
%         out:  The output of the algorithm which depends on the Output
%               option. If not specified, the options.Output=1 is adopted, i.e. the estimate of the unknown
%               vector, after the processing af all the available observations, is
%               returned. (see spaloptions help for more options.)
%     -------------------------------------------------------------------------------
%
% REFERENCES
%
%[1] Y. Kopsinis, K. Slavakis, S. Theodoridis, "Online Sparse System Identification and Signal Reconstruction Using
%Projections Onto Weighted $ell_{1}$ Balls," Signal Processing, IEEE Transactions on , vol.59, no.3, pp.936-952, March 2011
%doi: 10.1109/TSP.2010.2090874
%
% Ver. 1.0
% Yannis Kopsinis, 10/02/2011
% kopsinis@ieee.org



options = spaloptions(varargin);


ok = checkparamok(options.Method,options);
if ok==0
    error('Please, specify the parameters above (see spaloptions.m help)')
end

if isempty(options.Output)
    options.Output=1;
end

sX=size(X);
N=length(y); 

if sX(1)==1 || sX(2)==1
    if max(sX)~=L+N-1
        error('Vector X should have L+N-1 components')
    end
    if sX(1)==1
        X=X';
    end
else
    if sX(1)~=N || sX(2)~=L
        error('Matrix X should be NxL')
    end
end

if size(y,2)>1
    y=y';
end

method=options.Method;

%%%%%%%   Initialization   %%%%%%%

if options.Verbose
    fprintf('  --- Initialization --- \n')
end

[out]=method(y,X,L,options);



